//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//Clippoly.cpp
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//
//A  few  functions  to perform
//Sutherland-Hodgman  X,Y poly-
//gon  clipping  with  accurate
//results. 
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//Author: John De Goes
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//
//NOTE: Parameters invertex and outvertex can NOT be the same variable, as
//outvertex is modified dynamically.  The variables inlength and outlength 
//must also be different variables.  When the function returns,  outlength
//will hold the new number of vertices the clipped polygon has.
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//Before using this file(s), see Readme.txt.
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
//
//Example code:
//
//Clip_Right(vert1,vert2,length1,length2);
//Clip_Left(vert2,vert1,length2,length1);
//Clip_Top(vert1,vert2,length1,length2);
//Clip_Bottom(vert2,vert1,length2,length1);
//
//if (length1) 
//   Draw_Polygon(vert1,length1);
//
//
//Where Draw_Polygon is your own polygon drawing function.
//
//~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~

#include "Clippoly.hpp"

void Clip_Left(const Screen_Vertex *invertex, Screen_Vertex *outvertex, int inlength, int &outlength)
	{
	Screen_Vertex s,p,i;
	outlength=0;
	s=invertex[inlength-1];
	for (unsigned int j=0; j<inlength; j++)
		 {
		 p=invertex[j];
		 if (p.X >= XMIN)
			 {
			 if (s.X >= XMIN)
				 {
				 outvertex[outlength++]=p;
				 }
				 
			 else {
					//p is inside clip boundry, but s is outside clip boundry
					float slope=(float)(p.Y - s.Y) / (float)(p.X - s.X);
					i.X = XMIN;
					i.Y=s.Y + (float)slope * (float)(XMIN - s.X);
					outvertex[outlength++]=i;
					outvertex[outlength++]=p;
					}
			 }
		 else {
				if (s.X >= XMIN)
					{
					//s is inside clip boundry, but p is outside clip boundry
					float slope=(float)(s.Y - p.Y) / (float)(s.X - p.X);
					i.X = XMIN;
					i.Y=p.Y + (float)slope * (float)(XMIN - p.X);
					outvertex[outlength++]=i;
					}
				}
		 s=p;
		 }
	}

void Clip_Top(const Screen_Vertex *invertex, Screen_Vertex *outvertex, int inlength, int &outlength)
	{
	Screen_Vertex s,p,i;
	outlength=0;
	s=invertex[inlength-1];
	for (unsigned int j=0; j<inlength; j++)
		 {
		 p=invertex[j];
		 if (p.Y >= YMIN)
			 {
			 if (s.Y >= YMIN)
				 {
				 //p and s inside clip boundry
				 outvertex[outlength++]=p;
				 }
			 else {
					//p inside clip boundry, but s outside clip boundry
					float slope=(float)(p.X - s.X) / (float)(p.Y - s.Y);
					i.X=s.X + (float)(YMIN - s.Y) * (float)slope;
					i.Y=YMIN;
					outvertex[outlength++]=i;
					outvertex[outlength++]=p;
					}
			 }
		 else {
				if (s.Y >= YMIN)
					{
					//s inside clip boundry, but p outside clip boundry
					float slope=(float)(s.X - p.X) / (float)(s.Y - p.Y);
					i.X=p.X + (float)(YMIN - p.Y) * (float)slope;
					i.Y=YMIN;
					outvertex[outlength++]=i;
					}
				}
		 s=p;
		 }
	}

void Clip_Right(const Screen_Vertex *invertex, Screen_Vertex *outvertex, int inlength, int &outlength)
	{
	Screen_Vertex s,p,i;
	outlength=0;
	s=invertex[inlength-1];
	for (unsigned int j=0; j<inlength; j++)
		 {
		 p=invertex[j];
		 if (p.X <= XMAX)
			 {
			 if  (s.X <= XMAX)
				 {
				 outvertex[outlength++]=p;
				 }
			 else {
					//p inside clip boundry, but s outside clip boundry
					float slope=(float)(p.Y - s.Y) / (float)(p.X - s.X);
					i.X=XMAX;
					i.Y=s.Y + (float)slope * (float)(XMAX - s.X);
					outvertex[outlength++]=i;
					outvertex[outlength++]=p;
					}
			 }
		 else {
				if (s.X <= XMAX)
					{
					//s inside clip boundry, but p outside clip boundry
					float slope=(float)(s.Y - p.Y) / (float)(s.X - p.X);
					i.X=XMAX;
					i.Y=p.Y + (float)slope * (float)(XMAX - p.X);
					outvertex[outlength++]=i;
					}
				}
		 s=p;
		 }
	}

void Clip_Bottom(const Screen_Vertex *invertex, Screen_Vertex *outvertex, int inlength, int &outlength)
	{
	Screen_Vertex s,p,i;
	outlength=0;
	s=invertex[inlength-1];
	for (unsigned int j=0; j<inlength; j++)
		 {
		 p=invertex[j];
		 if (p.Y <= YMAX)
			 {
			 if (s.Y <= YMAX)
				 {
				 outvertex[outlength++]=p;
				 }     
			 else {
					//p inside clip boundry, but s outside clip boundry
					float slope=(float)(p.X - s.X) / (float)(p.Y - s.Y);
					i.X=s.X + (float)(YMAX - s.Y) * (float)slope;
					i.Y=YMAX;
					outvertex[outlength++]=i;
					outvertex[outlength++]=p;
					}
			 }
		 else {
				if (s.Y <= YMAX)
					{
					//s inside clip boundry, but p outside clip boundry
					float slope=(float)(s.X - p.X) / (float)(s.Y - p.Y);
					i.X=p.X + (float)(YMAX - p.Y) * (float)slope;
					i.Y=YMAX;
					outvertex[outlength++]=i;
					}
				}
		 s=p;
		 }
	}